# coding: utf-8
from __future__ import absolute_import

import json

from m3.actions import ActionPack
from m3.actions import ControllerCache

from m3_ext.ui.app_ui import BaseDesktopElement
from m3_ext.ui.app_ui import DesktopLoader
from m3_ext.ui.app_ui import DesktopModel
from m3_ext.ui.app_ui import MenuSeparator


class DesktopProcessor(object):

    @classmethod
    def filter_factory(cls, request, place):
        u"""
        Возвращает функцию фильтрации элементов рабочего стола:
        f(DesktopShortcut) -> bool.
        """

        def filter_by_permissions(elem):
            """
            Возвращает True, если у ползователя есть права на пак элемента.
            Работет только с DesktopShortcut'ами - у них есть атрибут pack,
            остальные же элементы - отображаются всегда
            :param elem: элемент рабочего стола
            :type elem: DesktopShortcut
            :return: has_perm - наличие права доступа к элементу
            :rtype: bool
            """
            pack = getattr(elem, 'pack', None)
            if pack is None or isinstance(pack, ActionPack):
                return True
            else:
                return pack.has_perm(request)
        return filter_by_permissions

    @classmethod
    def _dump_element(cls, obj):
        """
        Возвращает UI-элемент в сериализованном виде
        :param obj: элемент рабочего стола
        :type obj: MenuSeparator / BaseDesktopElement
        """
        if isinstance(obj, MenuSeparator):
            result = '-'
        elif isinstance(obj, BaseDesktopElement):
            result = {
                'text': obj.name,
                'icon': obj.icon,
                'extra': {}
            }
            if obj.has_subitems:
                result['items'] = list(obj.subitems)
            else:
                result['url'] = obj.url
                context = getattr(obj, 'context', {})
                if context:
                    result['context'] = context
                else:
                    result['context'] = {}
        else:
            raise TypeError("%r is not JSON-able!" % obj)
        return result

    @classmethod
    def _get_desktop(cls, request):
        """
        Формирует элементы Рабочего Стола
        :param request: request
        :type request: Request
        """
        desktop_model = DesktopModel(request, cls.filter_factory)
        ControllerCache.populate()
        DesktopLoader._success = False
        if hasattr(request, 'user'):
            DesktopLoader.populate(request.user, desktop=desktop_model)
        else:
            DesktopLoader.populate_desktop(desktop=desktop_model)

        return {
            'desktopItems': list(desktop_model.desktop.subitems),
            'menuItems': list(desktop_model.start_menu.subitems),
            'topToolbarItems': list(desktop_model.toptoolbar.subitems),
            'toolboxItems': list(desktop_model.toolbox.subitems),
        }

    @classmethod
    def process(cls, request):
        """
        Добавляет в контекст элементы Рабочего Стола
        :param request: request
        :type request: Request
        """
        desktop = cls._get_desktop(request)
        return {
            'desktop': json.dumps(
                desktop, indent=2, default=cls._dump_element,
                ensure_ascii=False),
            # TODO: переделать на JS!
            'desktop_icons': [
                (idx, i.name, i.icon)
                for (idx, i) in enumerate(desktop['desktopItems'], 1)
            ]
        }



# coding: utf-8
from __future__ import absolute_import

from m3_ext.ui import panels

from .base import BaseExtWindow


#==============================================================================
class BaseExtListWindow(BaseExtWindow):
    """
    Базовое окно со списком записей.
    """
    def __init__(self, *args, **kwargs):
        super(BaseExtListWindow, self).__init__(*args, **kwargs)

        #self.template = 'ext-windows/ext-window.js'

        self.layout = 'border'
        self.width = 800
        self.height = 600
        self.maximized = True

        # грид, который будем все для нас делать
        self.grid = panels.ExtObjectGrid(region='center')
        self.items.append(self.grid)

    # Данный код должен находится в базовом классе, но т.к. не вcе шаблоны
    # переведены на новый рендеринг, остается пока в каждом
    def render(self):
        assert getattr(self, '_ext_name'), (
            'Class %s is not define "_ext_name"' % self.__class__.__name__
        )

        self.pre_render()

        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError:
            raise Exception('Some attribute is not unicode')
        except Exception as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        res = '%(ext_name)s({%(base_config)s},{%(params)s})' % {
            'ext_name': self._ext_name,
            'base_config': base_config,
            'params': params
        }

        return 'new %s' % res if not self._is_function_render else res
